@@ -7,7 +7,7 @@ use vars qw($VERSION $BASE_EXC_CLASS %CLASSES);
 
 BEGIN { $BASE_EXC_CLASS ||= 'Exception::Class::Base'; }
 
-$VERSION = '1.13';
+$VERSION = '1.14';
 
 sub import
 {
@@ -15,13 +15,24 @@ sub import
 
     local $Exception::Class::Caller = caller();
 
+    my %c;
+
     my %needs_parent;
- MAKE_CLASSES:
     while (my $subclass = shift)
     {
 	my $def = ref $_[0] ? shift : {};
 	$def->{isa} = $def->{isa} ? ( ref $def->{isa} ? $def->{isa} : [$def->{isa}] ) : [];
 
+        $c{$subclass} = $def;
+    }
+
+    # We need to sort by length because if we check for keys in the
+    # Foo::Bar:: stash, this creates a "Bar::" key in the Foo:: stash!
+ MAKE_CLASSES:
+    foreach my $subclass ( sort { length $a <=> length $b } keys %c )
+    {
+        my $def = $c{$subclass};
+
 	# We already made this one.
 	next if $CLASSES{$subclass};
 
@@ -174,7 +185,7 @@ BEGIN
     __PACKAGE__->mk_classdata('NoRefs');
     *NoObjectRefs = \&NoRefs;
 
-    __PACKAGE__->NoRefs(0);
+    __PACKAGE__->NoRefs(1);
 
     sub Fields { () }
 }
@@ -366,20 +377,20 @@ Exception::Class - A module that allows you to declare real exception classes in
   eval { MyException->throw( error => 'I feel funny.'; };
 
   # catch
-  if ( $@ && ref $@ && $@->isa('MyException') )
+  if ( UNIVERSAL::isa( $@, 'MyException' ) )
   {
      warn $@->error, "\n, $@->trace->as_string, "\n";
      warn join ' ',  $@->euid, $@->egid, $@->uid, $@->gid, $@->pid, $@->time;
 
      exit;
   }
-  elsif ( $@->isa('ExceptionWithFields') )
+  elsif ( UNIVERSAL::isa( $@, 'ExceptionWithFields' ) )
   {
      $@->quixotic ? do_something_wacky() : do_something_sane();
   }
   else
   {
-     $@->rethrow;
+     ref $@ ? $@->rethrow : die $@;
   }
 
   # use an alias - without parens subroutine name is checked at
